@@ -41,8 +41,9 @@ sub _parse_fields {
         }
 
         my $v = shift @fields;
-        my @values = ( ref $v and !blessed $v )? @$v : ( "$v" );
-        push @new_fields, map { WebService::Solr::Field->new( $f => $_ ) } @values;
+        my @values = ( ref $v and !blessed $v ) ? @$v : ( "$v" );
+        push @new_fields,
+            map { WebService::Solr::Field->new( $f => $_ ) } @values;
     }
 
     return @new_fields;
@@ -60,7 +61,7 @@ sub values_for {
 
 sub to_xml {
     my $self = shift;
-    my $gen  = XML::Generator->new( ':std', escape => 'always,even-entities' );
+    my $gen = XML::Generator->new( ':std', escape => 'always,even-entities' );
     my %attr = ( $self->boost ? ( boost => $self->boost ) : () );
 
     return $gen->doc( \%attr, map { $_->to_xml } $self->fields );
@@ -137,7 +138,7 @@ Kirk Beers E<lt>kirk.beers@nald.caE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2008 National Adult Literacy Database
+Copyright 2008-2009 National Adult Literacy Database
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
